package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.PayPriceUtils;
import com.community.huiju.dao.*;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.Bill;
import com.community.huiju.model.BillInvoice;
import com.community.huiju.model.TaUser;
import com.community.huiju.model.TpBuildingOwnerInfo;
import com.community.huiju.service.IBillInvoiceService;
import com.google.common.collect.Maps;
import org.apache.poi.hssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.SimpleFormatter;
import java.util.stream.Collectors;

/**
 * <p>
 * 缴费单 服务实现类
 * </p>
 *
 * @author weiximei
 * @since 2019-02-13
 */
@Service
public class BillInvoiceServiceImpl extends ServiceImpl<BillInvoiceMapper, BillInvoice> implements IBillInvoiceService {

    /** 缴费项表 **/
    @Autowired
    private BillMapper billMapper;

    /** 缴费单 **/
    @Autowired
    private BillInvoiceMapper billInvoiceMapper;

    /** 账单流水表 **/
    @Autowired
    private BillStatementMapper billStatementMapper;

    @Autowired
    private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;

    @Autowired
    private TaUserMapper taUserMapper;

    @Override
    public ResponseBean getBillList(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(parameter);

        Integer pageNum = jsonObject.getInteger("pageNum");
        Integer pageSize = jsonObject.getInteger("pageSize");
        // 缴费项ID
        Integer billId = jsonObject.getInteger("billId");
        // 期
        String phase = jsonObject.getString("phase");
        // 栋
        String building = jsonObject.getString("building");
        // 单元
        String unit = jsonObject.getString("unit");
        // 户号
        String roomNo = jsonObject.getString("roomNo");
        // 业主姓名
        String ownerName = jsonObject.getString("ownerName");
        // 缴费状态
        Integer billStatus = jsonObject.getInteger("billStatus");
        // 缴费人姓名
        String payName = jsonObject.getString("payName");

        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", billId);
        map.put("phase", phase);
        map.put("building", building);
        map.put("unit", unit);
        map.put("roomNo", roomNo);
        map.put("ownerName", ownerName);
        map.put("billStatus", billStatus);
        map.put("payName", payName);
        map.put("communityId", userElement.getCommunityId());


        Page<BillInvoice> page = new Page<>();
        page.setCurrent(pageNum);
        page.setSize(pageSize);
        IPage<BillInvoice> invoiceIPage = billInvoiceMapper.selectBillInvoice(page, map);

        // 缴费项目
        Bill bill = billMapper.selectById(billId);

        Map<String, Object> respMap = Maps.newHashMap();
        respMap.put("pageNum", invoiceIPage.getCurrent());
        respMap.put("pageSize", invoiceIPage.getSize());
        respMap.put("total", invoiceIPage.getTotal());
        respMap.put("list", invoiceIPage.getRecords());
        respMap.put("billName", bill.getBillName());
        respMap.put("billExplain", bill.getBillExplain());
        respMap.put("endDate", bill.getEndDate());

        responseBean.addSuccess(respMap);

        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean deleteBillInvoice(List<Integer> ids, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        if (null == userElement) {
            responseBean.addError("请登录后操作");
            return responseBean;
        }

        ids.stream().forEach(e -> {
            BillInvoice billInvoice = billInvoiceMapper.selectById(e);
            if (!"0".equals(billInvoice.getBillStatus())) {
                throw new WisdomException("费用已缴，无法删除");
            }
            billInvoiceMapper.deleteById(e);
        });

        responseBean.addSuccess("操作成功");
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean offlinePayment(List<Integer> ids, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        if (null == userElement) {
            responseBean.addError("请登录后操作");
            return responseBean;
        }

        ids.stream().forEach(e -> {
            BillInvoice billInvoice = billInvoiceMapper.selectById(e);
            if ("1".equals(billInvoice.getBillStatus())) {
                throw new WisdomException("费用已缴，无法更改");
            }

            billInvoice.setBillStatus("2");
            billInvoiceMapper.updateById(billInvoice);
        });

        responseBean.addSuccess("操作成功");
        return responseBean;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean addTempBillInvoice(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        // 收费项ID
        Integer billId = jsonObject.getInteger("billId");
        // 区/期
        String phase = jsonObject.getString("phase");
        // 栋
        String building = jsonObject.getString("building");
        // 单元
        String unit = jsonObject.getString("unit");
        // 户号
        String roomNo = jsonObject.getString("roomNo");
        // 催缴日期
        // Date worthOf = jsonObject.getDate("worthOf");
        // 收费单说明
        String billInvoiceExplain = jsonObject.getString("billInvoiceExplain");
        // 收费金额
        Integer payPrice = jsonObject.getInteger("payPrice");

        // 楼栋资料库
        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phase", phase);
        queryWrapper.eq("building", building);
        queryWrapper.eq("unit", unit);
        queryWrapper.eq("room_no", roomNo);
        queryWrapper.eq("community_id", userElement.getCommunityId());
        TpBuildingOwnerInfo buildingOwnerInfo = tpBuildingOwnerInfoMapper.selectOne(queryWrapper);

        // 查询app用户id
        QueryWrapper<TaUser> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("community_id", userElement.getCommunityId());
        userQueryWrapper.eq("login_name", buildingOwnerInfo.getOwnerTel());
        TaUser taUser = taUserMapper.selectOne(userQueryWrapper);


        BillInvoice billInvoice = new BillInvoice();
        billInvoice.setBillId(billId);
        billInvoice.setCommunityId(userElement.getCommunityId());
        billInvoice.setBillInvoiceExplain(billInvoiceExplain);
        billInvoice.setPayPrice(PayPriceUtils.yuanConversionPoints(payPrice));
        billInvoice.setBuildingOwnerInfoId(buildingOwnerInfo.getId());
        billInvoice.setTaUserId(taUser.getId());
        billInvoice.setBillStatus("0");
        billInvoice.setCreateUser(userElement.getId());
        billInvoice.setCreateDate(new Date());
        billInvoice.setStatus(0);

        int result = billInvoiceMapper.insert(billInvoice);
        if (result <= 0) {
            throw new WisdomException("添加失败");
        }

        responseBean.addSuccess("操作成功");
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean updateBillInvoiceStatus(List<Integer> ids, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();

        List<BillInvoice> billInvoices = ids.stream().map(e -> {
            BillInvoice billInvoice = billInvoiceMapper.selectById(e);
            billInvoice.setStatus(1);
            billInvoice.setUpdateDate(new Date());
            billInvoice.setUpdateUser(userElement.getId());
            return billInvoice;
        }).collect(Collectors.toList());

        boolean b = this.updateBatchById(billInvoices);
        if (b == false) {
            throw new WisdomException("操作失败");
        }
        responseBean.addSuccess("操作成功");
        return responseBean;
    }

    @Override
    public ResponseBean getTempBillInvoice(Integer billId,UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", billId);
        map.put("communityId", userElement.getCommunityId());

        List<BillInvoice> billInvoices = billInvoiceMapper.selectBillInvoiceTemp(map);

        responseBean.addSuccess(billInvoices);
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean deleteTempBillInvoice(Integer billId, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        if (null == userElement) {
            responseBean.addError("未登录,无法操作!");
            return responseBean;
        }

        BillInvoice billInvoice = billInvoiceMapper.selectById(billId);
        if (billInvoice.getStatus() != 0) {
            responseBean.addError("只能删除作废的收费单");
            return responseBean;
        }
        int result = billInvoiceMapper.deleteById(billId);
        if (result <= 0) {
            throw new WisdomException("操作失败");
        }
        responseBean.addSuccess("操作成功");
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean updateBillInvoiceIdPayPrice(Integer billId, Integer payPrice, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        BillInvoice billInvoice = billInvoiceMapper.selectById(billId);
        billInvoice.setPayPrice(PayPriceUtils.yuanConversionPoints(payPrice));
        billInvoice.setUpdateUser(userElement.getId());
        billInvoice.setUpdateDate(new Date());

        int result = billInvoiceMapper.updateById(billInvoice);
        if (result <= 0) {
            throw new WisdomException("操作失败");
        }
        responseBean.addSuccess("操作成功");
        return responseBean;
    }

    @Override
    public ResponseBean exportExcel(UserElement userElement, Integer billId) {
        ResponseBean responseBean = new ResponseBean();
        String [] title = new String[] { "收费单号","房屋信息","业主姓名","收费单说明","金额(元)","流水","缴费状态","缴费人","缴费时间","新建时间","新建人","修改人","修改时间" };

        HSSFWorkbook excel = excel(title, userElement, billId,null);
        responseBean.addSuccess(excel);
        return responseBean;
    }

    /**
     * 表格
     * @param title
     * @param workbook
     * @return
     */
    private HSSFWorkbook excel(String[] title, UserElement userElement, Integer billId, HSSFWorkbook workbook) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

        if (null == workbook) {
            workbook = new HSSFWorkbook();
        }
        HSSFSheet sheet = workbook.createSheet("缴费单");
        // 行数标记
        int rowNum = 0;
        // 创建一行,用作标题
        HSSFRow row = sheet.createRow(rowNum);
        // 创建样式
        HSSFCellStyle style = workbook.createCellStyle();
        // 居中
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);

        // 创建列对象
        HSSFCell cell = null;

        // 写入表头
        for (int i = 0; i<title.length; i++) {
            cell = row.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
        }

        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", billId);
        map.put("communityId", userElement.getCommunityId());

        Page<BillInvoice> page = new Page<>();
        page.setCurrent(1);
        page.setSize(500);
        IPage<BillInvoice> billInvoiceIPage = billInvoiceMapper.selectBillInvoiceEffective(page, map);
        List<BillInvoice> records = billInvoiceIPage.getRecords();


        // 写入内容
        for (int i = 1; i<= billInvoiceIPage.getPages(); i++) {
            for (int t = 0; t<records.size(); t++) {
                row = sheet.createRow(++rowNum);
                // 收费单号
                cell = row.createCell(0);
                cell.setCellValue(records.get(t).getId());
                cell.setCellStyle(style);
                // 房屋信息
                cell = row.createCell(1);
                cell.setCellValue(records.get(t).getPhase() + records.get(t).getBuilding() + records.get(t).getUnit() + records.get(t).getRoomNo());
                cell.setCellStyle(style);
                // 业主姓名
                cell = row.createCell(2);
                cell.setCellValue(records.get(t).getOwnerName());
                cell.setCellStyle(style);
                // 收费单说明
                cell = row.createCell(3);
                cell.setCellValue(records.get(t).getBillInvoiceExplain());
                cell.setCellStyle(style);
                // 金额(元)
                cell = row.createCell(4);
                double price = Double.valueOf(records.get(t).getPayPrice()) / 100.00;
                cell.setCellValue(price);
                cell.setCellStyle(style);
                if (null != records.get(t).getBillStatement()) {
                    // 流水
                    cell = row.createCell(5);
                    cell.setCellValue(records.get(t).getBillStatement());
                    cell.setCellStyle(style);
                }
                // 缴费状态
                cell = row.createCell(6);
                cell.setCellValue("0".equals(records.get(t).getBillStatus()) ? "未交费" : "1".equals(records.get(t).getBillStatus()) ? "已线上缴费" : "已线下缴费");
                cell.setCellStyle(style);
                // 缴费人
                cell = row.createCell(7);
                cell.setCellValue(records.get(t).getPayName());
                cell.setCellStyle(style);
                if (null != records.get(t).getPayDate()) {
                    // 缴费时间
                    cell = row.createCell(8);
                    cell.setCellValue(dateFormat.format(records.get(t).getPayDate()));
                    cell.setCellStyle(style);
                }
                // 新建时间
                cell = row.createCell(9);
                cell.setCellValue(dateFormat.format(records.get(t).getCreateDate()));
                cell.setCellStyle(style);
                // 新建人
                cell = row.createCell(10);
                cell.setCellValue(records.get(t).getCreateUserName());
                cell.setCellStyle(style);
                // 修改人
                cell = row.createCell(11);
                cell.setCellValue(records.get(t).getUpdateUserName());
                cell.setCellStyle(style);
                if (null != records.get(t).getUpdateDate()) {
                    // 修改时间
                    cell = row.createCell(12);
                    cell.setCellValue(dateFormat.format(records.get(t).getUpdateDate()));
                    cell.setCellStyle(style);
                }
            }
            page.setCurrent(i);
            page.setSize(500);
            billInvoiceIPage = billInvoiceMapper.selectBillInvoiceEffective(page, map);
            records = billInvoiceIPage.getRecords();

        }

        return workbook;
    }


    @Override
    public ResponseBean getInvoiceInvalid(UserElement userElement, String parameter) {
        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        Integer pageNum = jsonObject.getInteger("pageNum");
        Integer pageSize = jsonObject.getInteger("pageSize");
        Integer billId = jsonObject.getInteger("billId");

        Bill bill = billMapper.selectById(billId);

        Page page = new Page();
        page.setCurrent(pageNum);
        page.setSize(pageSize);

        QueryWrapper<BillInvoice> billInvoiceQueryWrapper = new QueryWrapper<>();
        billInvoiceQueryWrapper.eq("bill_id", billId);
        billInvoiceQueryWrapper.eq("community_id", userElement.getCommunityId());

        IPage<BillInvoice> billInvoiceIPage = billInvoiceMapper.selectPage(page, billInvoiceQueryWrapper);
        List<BillInvoice> records = billInvoiceIPage.getRecords();
        records.forEach(e->{
            TpBuildingOwnerInfo buildingOwnerInfo = tpBuildingOwnerInfoMapper.selectById(e.getBuildingOwnerInfoId());
            e.setOwnerName(buildingOwnerInfo.getOwnerName());
            e.setPhase(buildingOwnerInfo.getPhase());
            e.setBuilding(buildingOwnerInfo.getBuilding());
            e.setUnit(buildingOwnerInfo.getUnit());
            e.setLevel(buildingOwnerInfo.getLevel());
        });

        Map<String, Object> map = Maps.newHashMap();
        map.put("bill", bill);
        map.put("billInvoice", records);
        map.put("total", billInvoiceIPage.getTotal());
        map.put("pageNum", billInvoiceIPage.getCurrent());
        map.put("pageSize", billInvoiceIPage.getSize());

        responseBean.addSuccess(map);
        return responseBean;
    }
}
